@@ -28,11 +28,9 @@
 
 using System;
 using System.IO;
-using System.Text;
 using System.Xml;
 using System.Collections.Generic;
 
-using Banshee.Base;
 using Banshee.Collection.Database;
 
 using Hyena;
@@ -67,6 +65,7 @@ namespace Banshee.LiveRadio.Plugins
             if (!Int32.TryParse(HttpTimeoutEntry.Get (), out http_timeout_seconds))
                 http_timeout_seconds = 20;
             proxy_url = ProxyUrlEntry.Get ();
+            SetWebIcon ("http://my.magnatune.com/favicon.ico");
         }
 
         /// <summary>
@@ -159,9 +158,9 @@ namespace Banshee.LiveRadio.Plugins
                     RaiseErrorReturned ("XML Parse Error", ex.Message);
                     continue;
                 }
-                
+
             }
-            
+
             new_genres.Sort ();
             genres = new_genres;
 
@@ -439,5 +438,5 @@ namespace Banshee.LiveRadio.Plugins
 
 
     }
-    
+
 }
