@@ -1,9 +1,9 @@
-//  
+//
 // Author:
 //   Christian Martellini <christian.martellini@gmail.com>
 //
 // Copyright (C) 2009 Christian Martellini
-// 
+//
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
 // "Software"), to deal in the Software without restriction, including
@@ -22,10 +22,11 @@
 // LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-// 
+//
 
 using System;
 using System.Collections.Generic;
+using System.Linq;
 
 using Banshee.Networking;
 using Banshee.ServiceStack;
@@ -62,7 +63,8 @@ namespace Banshee.Lyrics
         public event LoadFinishedEventHandler LoadFinished;
         public event LoadStartedEventHandler LoadStarted;
 
-        private List < ILyricsSource > sourceList;
+        private readonly List <SourceData> sources = new List<SourceData> ();
+        private readonly Lyrc lyrc;
 
         private LyricsCache cache = new LyricsCache ();
 
@@ -70,13 +72,12 @@ namespace Banshee.Lyrics
 
         private LyricsManager () : base()
         {
-            sourceList = new List<ILyricsSource> ();
-            sourceList.Add (new Lyrc ());
-            sourceList.Add (new LeosLyrics ());
-            sourceList.Add (new Lyriki ());
-            sourceList.Add (new AutoLyrics ());
-            sourceList.Add (new LyricWiki ());
-            sourceList.Add (new LyricsPlugin ());
+            sources.Add (new SourceData (lyrc = new Lyrc ()));
+            sources.Add (new SourceData (new LyricWiki ()));
+            sources.Add (new SourceData (new LeosLyrics ()));
+            sources.Add (new SourceData (new Lyriki ()));
+            sources.Add (new SourceData (new AutoLyrics ()));
+            sources.Add (new SourceData (new LyricsPlugin ()));
         }
 
         internal static LyricsManager Instance {
@@ -93,7 +94,7 @@ namespace Banshee.Lyrics
         }
 
         /*
-         * Get the lyricss for the current track
+         * Get the lyrics for the current track
          */
         public void FetchLyrics (TrackInfo track)
         {
@@ -121,7 +122,7 @@ namespace Banshee.Lyrics
                         return;
                     }
                 } catch (Exception e) {
-                    Hyena.Log.DebugException (e);
+                    Log.DebugException (e);
                     error = e.Message;
                 }
 
@@ -136,31 +137,38 @@ namespace Banshee.Lyrics
                 return null;
             }
 
-            //check if the netowrk is up
+            //check if the network is up
             if (!ServiceManager.Get<Banshee.Networking.Network> ().Connected) {
                 throw new Exception ("You don't seem to be connected to internet.<br>Check your network connection.");
             }
-            
-            //download the lyricss
+
+            //download the lyrics
             string lyrics = null;
-            foreach (ILyricsSource source in sourceList) {
+            foreach (var source in GetSources (SourceData.LyricsSelector)) {
+                bool found = false;
                 try {
-                    lyrics = source.GetLyrics (track.ArtistName, track.TrackTitle);
+                    lyrics = source.Source.GetLyrics (track.ArtistName, track.TrackTitle);
+                    found = IsLyricsOk (lyrics);
                 } catch (Exception e) {
                     Log.Exception (e);
                     continue;
+                } finally {
+                    source.IncrementLyrics (found);
                 }
-                
-                if (IsLyricsOk (lyrics)) {
-                    lyrics = AttachFooter (Utils.ToNormalString(lyrics), source.Credits);
-                    break;
+
+                if (found) {
+                    lyrics = AttachFooter (Utils.ToNormalString(lyrics), source.Source.Credits);
+                    Log.DebugFormat ("Fetched lyrics from {0} for {1} - {2}",
+                        source.Source.Name, track.ArtistName, track.TrackTitle);
+                    return lyrics;
                 }
             }
 
-            return lyrics;
+            Log.DebugFormat ("Couldn't find lyrics for {0} - {1}", track.ArtistName, track.TrackTitle);
+            return null;
         }
 
-        public void UpdateDB (TrackInfo track, string lyrics) 
+        public void UpdateDB (TrackInfo track, string lyrics)
         {
             int track_id = ServiceManager.SourceManager.MusicLibrary.GetTrackIdForUri (track.Uri.AbsoluteUri);
             ServiceManager.DbConnection.Execute (
@@ -177,17 +185,16 @@ namespace Banshee.Lyrics
             LoadStarted (this, null);
 
             /*obtain absolute url for the lyrics */
-            Lyrc lyrc_server = (Lyrc) sourceList[0];
-            if (!url.Contains (lyrc_server.Url)) {
-                url = lyrc_server.Url + "/" + url;
+            if (!url.Contains (lyrc.Url)) {
+                url = lyrc.Url + "/" + url;
             }
 
             /* get the lyrics from lyrc */
-            string lyrics = lyrc_server.GetLyrics (url);
+            string lyrics = lyrc.GetLyrics (url);
 
             LyricsManager.Instance.UpdateDB (ServiceManager.PlayerEngine.CurrentTrack, lyrics);
             if ( IsLyricsOk (lyrics)) {
-                lyrics = AttachFooter (lyrics, lyrc_server.Credits);
+                lyrics = AttachFooter (lyrics, lyrc.Credits);
                 Save(ServiceManager.PlayerEngine.CurrentTrack, lyrics);
             }
 
@@ -196,15 +203,32 @@ namespace Banshee.Lyrics
 
         private string GetSuggestions (TrackInfo track)
         {
-            //Obtain suggestions from Lyrc
-            ILyricsSource lyrc_server = sourceList[0];
+            string suggestions = null;
+            foreach (var source in GetSources (SourceData.SuggestionsSelector)) {
+                bool found = false;
+                try {
+                    suggestions = source.Source.GetSuggestions (track.ArtistName, track.TrackTitle);
+                    found = !String.IsNullOrEmpty (suggestions);
+                } catch (Exception e) {
+                    Log.Exception (e);
+                    continue;
+                } finally {
+                    source.IncrementSuggestions (found);
+                }
 
-            return lyrc_server.GetSuggestions (track.ArtistName, track.TrackTitle);
+                if (found) {
+                    Log.DebugFormat ("Fetched suggestions from {0} for {1} - {2}",
+                        source.Source.Name, track.ArtistName, track.TrackTitle);
+                    return suggestions;
+                }
+            }
+            Log.DebugFormat ("Couldn't find any suggestions for {0} - {1}", track.ArtistName, track.TrackTitle);
+            return null;
         }
 
         private bool IsLyricsOk (string l)
         {
-            return l != null && !l.Equals ("");
+            return !String.IsNullOrEmpty (l);
         }
 
         private bool LyricOutOfDate (TrackInfo track)
@@ -271,5 +295,54 @@ namespace Banshee.Lyrics
             track.FileModifiedStamp = Banshee.IO.File.GetModifiedTime (track.Uri);
             track.LastSyncedStamp = DateTime.Now;
         }
+
+        private IEnumerable<SourceData> GetSources (Func<SourceData, double> selector)
+        {
+            return sources.OrderBy<SourceData, double> (selector);
+        }
+
+        private class SourceData
+        {
+            public ILyricsSource Source { get; private set; }
+            public int LyricsTotal { get; private set; }
+            public int LyricsFound { get; private set; }
+            public int SuggestionsTotal { get; private set; }
+            public int SuggestionsFound { get; private set; }
+
+            public SourceData (ILyricsSource source)
+            {
+                Source = source;
+                LyricsTotal = 0;
+                LyricsFound = 0;
+                SuggestionsTotal = 0;
+                SuggestionsFound = 0;
+            }
+
+            public static double LyricsSelector (SourceData data)
+            {
+                return data.LyricsTotal > 0 ? 1d - (double)data.LyricsFound / data.LyricsTotal : 0d;
+            }
+
+            public static double SuggestionsSelector (SourceData data)
+            {
+                return data.SuggestionsTotal > 0 ? 1d - (double)data.SuggestionsFound / data.SuggestionsTotal : 0d;
+            }
+
+            public void IncrementLyrics (bool found)
+            {
+                LyricsTotal++;
+                if (found) {
+                    LyricsFound++;
+                }
+            }
+
+            public void IncrementSuggestions (bool found)
+            {
+                SuggestionsTotal++;
+                if (found) {
+                    SuggestionsFound++;
+                }
+            }
+        }
     }
 }
